/*
   Copyright (c) 1997 Christian Esken (chris@magicon.prima.ruhr.de)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
	$Id$

 	kmedia.cpp : the GUI part of the media master
		     The protocol resides in libmediatool.a
*/
	
extern "C" {
#include "mediatool.h"
}

#include "kmedia.h"

// Bitmaps from XMCD
#include "bitmaps/playpaus.xbm"
#include "bitmaps/stop.xbm"
#include "bitmaps/repeat.xbm"
#include "bitmaps/nexttrk.xbm"
#include "bitmaps/prevtrk.xbm"
#include "bitmaps/ff.xbm"
#include "bitmaps/rew.xbm"
#include "bitmaps/poweroff.xbm"
#include "bitmaps/eject.xbm"
#include "bitmaps/logo.xbm"

/****************************************************************************
		Global variables
 ****************************************************************************/

int32		*StatStatPtr;
int32		key;
char		RedisplayKeys = 0;
MdCh_KEYS	*KeysChunk;
MdCh_STAT	*StatChunk;

/****************************************************************************
				The GUI part 
*****************************************************************************/

/****************************************************************************
	
			W A R N I N G : THE CODE IS UGLY

 ****************************************************************************/

KMEDIA::KMEDIA( QWidget *parent, const char *name ) :
	QWidget( parent, name )
{
	blinker = new QTimer( this );	 
	timer = new QTimer( this );
	volTimer = new QTimer( this );
	

	initMediatool();
	drawPanel();
	loadBitmaps();
	initToolTip();
	statusPB->setPalette( QPalette( statusPB->colorGroup(), 
					statusPB->colorGroup(),
			      		statusPB->colorGroup() ) );
	statusPB->setEnabled( FALSE );
	connect( timer, 	SIGNAL(timeout()), 	SLOT(cdMode()) );
	connect( blinker, 	SIGNAL(timeout()), 	SLOT(blink()) );
/*	connect( volTimer,	SIGNAL(timeout()),	SLOT(volUpdate()) ); */
	connect( playPB, 	SIGNAL(clicked()), 	SLOT(playClicked()) );
	connect( stopPB, 	SIGNAL(clicked()), 	SLOT(stopClicked()) );
	connect( prevPB, 	SIGNAL(pressed()), 	SLOT(prevClicked()) );
	connect( nextPB, 	SIGNAL(pressed()), 	SLOT(nextClicked()) );
	connect( fwdPB, 	SIGNAL(pressed()), 	SLOT(fwdClicked()) );
	connect( bwdPB, 	SIGNAL(pressed()), 	SLOT(bwdClicked()) );
	connect( prevPB, 	SIGNAL(released()), 	SLOT(prevReleased()) );
	connect( nextPB, 	SIGNAL(released()), 	SLOT(nextReleased()) );
	connect( fwdPB, 	SIGNAL(released()), 	SLOT(fwdReleased()) );
	connect( bwdPB, 	SIGNAL(released()), 	SLOT(bwdReleased()) );
	connect( quitPB, 	SIGNAL(clicked()), 	SLOT(quitClicked()) );	
	connect( replayPB, 	SIGNAL(clicked()), 	SLOT(replayClicked()) );
	connect( ejectPB, 	SIGNAL(clicked()), 	SLOT(ejectClicked()) );
	connect( songListCB, 	SIGNAL(activated(int)), SLOT(trackSelected(int)));
/*	connect( volSB, 	SIGNAL(valueChanged(int)), SLOT(volChanged(int))); */
	connect( aboutPB, 	SIGNAL(clicked()), 	SLOT(aboutClicked()));
	timer->start( 50 );
	volTimer->start( 50 );
}

// Experimental ToolTips
void KMEDIA::initToolTip() 
{
#ifdef QTBETA
	QToolTip::add( trackTimeLCD, 	"Time elapsed" );
	QToolTip::add( trackLCD, 	"Current track" );
	QToolTip::add( playPB, 		"Play/Pause" );
	QToolTip::add( stopPB, 		"Stop" );
	QToolTip::add( replayPB, 	"Replay current" );
	QToolTip::add( songListCB, 	"track selection" );
	QToolTip::add( fwdPB, 		"Forward" );
	QToolTip::add( bwdPB, 		"Backward" );
	QToolTip::add( nextPB, 		"Next track" );
	QToolTip::add( prevPB, 		"Previous track" );
	QToolTip::add( quitPB, 		"Quit" );
	QToolTip::add( aboutPB, 	"About" );
	QToolTip::add( ejectPB, 	"Eject CD" );
	QToolTip::add( statusPB, 	"Status" );
	QToolTip::add( volSB, 		"Volume control" );
	QToolTip::add( volLA, 		"Volume level" );
#else
	printf( "No ToolTip in this version\n");
#endif
}

void KMEDIA::initMediatool() 
{
  MediaCon  m;

  MdConnectInit();
  MdConnectNew(&m);
  if ( m.shm_adr == NULL )
    dtfatal("Could not create media connection.\n");


  printf("The media id (talk id) for today is %i.\n", m.talkid);

  KeysChunk   = (MdCh_KEYS*)FindChunkData(m.shm_adr, "KEYS");
  StatChunk   = (MdCh_STAT*)FindChunkData(m.shm_adr, "STAT");
  StatStatPtr =  &(StatChunk->status);
}

// Create, place and set the name of a button
QPushButton *KMEDIA::makeButton( int x, int y, int w, int h, const char *n ) 
{
	QPushButton *pb = new QPushButton( n, this );
	pb->setGeometry( x, y, w, h );
	return pb;
}
	
// Make the Time LCD blink every 1/2 sec while pausing
void KMEDIA::blink()
{
	static bool state = TRUE;

	if (state) 
		setLCDColor( trackTimeLCD, green, black );
	else 
		setLCDColor( trackTimeLCD, black, black );
	state = !state;
}
		
// Widgets have to be placed by manually......
void KMEDIA::drawPanel() 
{
	int ix = 0;
	int iy = 0;
	int vol1=0;		// left


	const int WIDTH = 100;
	const int HEIGHT = 30;
	const int SBARWIDTH = 140;

	setCaption( "Kmedia" );
	aboutPB = makeButton( ix, iy, WIDTH, 2 * HEIGHT, "About" );

	ix = 0;
	iy += 2 * HEIGHT;
	ejectPB = makeButton( ix, iy, WIDTH, HEIGHT, "Eject" );

	iy += HEIGHT;
	quitPB = makeButton( ix, iy, WIDTH, HEIGHT, "Quit" );

	ix += WIDTH;
	iy = 0;
	trackLCD = new QLCDNumber( this );
	trackLCD->setGeometry( ix, iy, 2 * SBARWIDTH / 7 , 2 * HEIGHT ); 
	trackLCD->setFrameStyle( QFrame::NoFrame );
	setLCDColor( trackLCD, green, black );
	
	ix += 2 * SBARWIDTH / 7;
	trackTimeLCD = new QLCDNumber( this );
	trackTimeLCD->setGeometry( ix, iy, 5 * SBARWIDTH / 7, 2 * HEIGHT );
	trackTimeLCD->setFrameStyle( QFrame::NoFrame );
	setLCDColor( trackTimeLCD, green, black );

	ix = WIDTH;
	iy += 2 * HEIGHT;
	/*statusLA = new QLabel( this );
        statusLA->setFrameStyle( QFrame::Panel | QFrame::Raised );
        statusLA->setAlignment( AlignCenter );
        statusLA->setGeometry( ix, iy, 60, HEIGHT );
	statusLA->setFont( QFont( "helvetica", 12 ) );*/
	statusPB = makeButton( ix, iy, 60, HEIGHT, "" );
	statusPB->setFont( QFont( "helvetica", 12 ) );

	ix += 60;
	songListCB = new QComboBox( this );
	songListCB->setGeometry( ix, iy, 80, HEIGHT );
	songListCB->setFont( QFont( "helvetica", 12 ) );

	ix = WIDTH;
	iy += HEIGHT;
	// Volume control here
	volLA = new QLabel( this );
        volLA->setAlignment( AlignCenter );
        volLA->setGeometry( ix, iy, SBARWIDTH, HEIGHT / 2 );
        volLA->setFont( QFont( "helvetica", 12 ) );
	volLA->setNum( vol1 );

	iy += HEIGHT / 2 ;
	volSB = new QScrollBar( 0, 100, 1, 1, vol1, QScrollBar::Horizontal, this );
	volSB->setGeometry( ix, iy, SBARWIDTH, HEIGHT /2 );

	iy = 0;
	ix = WIDTH + SBARWIDTH;
	playPB = makeButton( ix, iy, WIDTH, HEIGHT, "Play/Pause" );

	iy += HEIGHT;
	stopPB = makeButton( ix, iy, WIDTH / 2, HEIGHT, "Stop" );

	ix += WIDTH / 2;
	replayPB = makeButton( ix, iy, WIDTH / 2, HEIGHT, "Replay" );

	ix = WIDTH + SBARWIDTH;
	iy += HEIGHT;
	bwdPB = makeButton( ix, iy, WIDTH / 2, HEIGHT, "Bwd" );

	ix += WIDTH / 2;
	fwdPB = makeButton( ix, iy, WIDTH / 2, HEIGHT, "Fwd" );

	ix = WIDTH + SBARWIDTH;
	iy += HEIGHT;
	prevPB = makeButton( ix, iy, WIDTH / 2, HEIGHT, "Prev" );

	ix += WIDTH / 2;
	nextPB = makeButton( ix, iy, WIDTH / 2, HEIGHT, "Next" );

	resize( WIDTH * 2 + SBARWIDTH, 4 * HEIGHT );
	
}

// Set up the bitmaps for the buttons
void KMEDIA::loadBitmaps() {
	QBitmap playBmp( playpause_width, playpause_height, playpause_bits,
			 TRUE );
	QBitmap stopBmp( stop_width, stop_height, stop_bits, TRUE );
	QBitmap prevBmp( prevtrk_width, prevtrk_height, prevtrk_bits, TRUE );
	QBitmap nextBmp( nexttrk_width, nexttrk_height, nexttrk_bits, TRUE );
	QBitmap replayBmp( repeat_width, repeat_height, repeat_bits, TRUE );
	QBitmap fwdBmp( ff_width, ff_height, ff_bits, TRUE );
	QBitmap bwdBmp( rew_width, rew_height, rew_bits, TRUE );
	QBitmap ejectBmp( eject_width, eject_height, eject_bits, TRUE );
	QBitmap quitBmp( poweroff_width, poweroff_height, poweroff_bits, 
			 TRUE );
	QBitmap aboutBmp( logo_width, logo_height, logo_bits, TRUE );

	playPB->setPixmap( playBmp );
	stopPB->setPixmap( stopBmp );
	prevPB->setPixmap( prevBmp );
	nextPB->setPixmap( nextBmp );
	replayPB->setPixmap( replayBmp );
	fwdPB->setPixmap( fwdBmp );
	bwdPB->setPixmap( bwdBmp );
	ejectPB->setPixmap( ejectBmp );
	quitPB->setPixmap( quitBmp );
	aboutPB->setPixmap( aboutBmp );
}

void KMEDIA::playClicked()
{
  /* play/pause/unpause */
  if ( KeysChunk->pause )
    {
      KeysChunk->pause = 0;
      EventCounterRaise(&(KeysChunk->play),1);
      statusPB->setText( "Playing" ); /* unpaused, playing */
    }
  else
    {
      if  ( *StatStatPtr & MD_STAT_PLAYING )
	{
	  KeysChunk->pause = 1;
	  statusPB->setText("Paused");
	}
      else
	{
	  EventCounterRaise(&(KeysChunk->play),1);
	  statusPB->setText("Playing");
	}
    }
}

void KMEDIA::stopClicked()
{
  EventCounterRaise(&(KeysChunk->stop),1);
}

void KMEDIA::prevClicked()
{
  EventCounterRaise(&(KeysChunk->prevtrack),1);
}

void KMEDIA::prevReleased()
{
  /* empty for now */
}

void KMEDIA::nextClicked()
{
  EventCounterRaise(&(KeysChunk->nexttrack),1);
}

void KMEDIA::nextReleased()
{
  /* empty for now */
}

void KMEDIA::fwdClicked()
{
  statusPB->setText("Forward");
  EventCounterRaise(&(KeysChunk->forward),1);
}

void KMEDIA::fwdReleased()
{
  /* empty for now */
}

void KMEDIA::bwdClicked()
{
  statusPB->setText("Backward");
  EventCounterRaise(&(KeysChunk->backward),1);
}

void KMEDIA::bwdReleased()
{
  /* empty for now */
}

void KMEDIA::quitClicked()
{
#ifdef QTBETA
  bool ok = QMessageBox::query( "Confirm", "Really Quit?", "Yes", "No" );
#else
  bool ok = TRUE;
#endif
  if (ok) {
    timer->stop();
    EventCounterRaise(&(KeysChunk->exit),1);
    qApp->quit();
  }
}

void KMEDIA::replayClicked()
{
  fprintf(stderr, "Replay\n");
}

void KMEDIA::ejectClicked()
{
  fprintf(stderr, "Eject (killing player). Experimental! \n");
  EventCounterRaise(&(KeysChunk->exit),1);
}

void KMEDIA::trackSelected( int trk )
{
  fprintf(stderr, "Track selected: %i\n",trk);
}

void KMEDIA::aboutClicked()
{
	QMessageBox::message( "About Kmedia 0.1", "Kmedia\n"\
			      "Written by Christian Esken, 1997\n"\
			      "Mediatool protocol and kmedia are (C)1997\n"\
			      "by Christian Esken. This program is in the GPL.\n"\
			      "Send bug reports to chris@magicon.prima.ruhr.de .",
			      "OK" );
}

// It runs every 1/2 sec checking the status of the media slave and updates
// the appropriate data.
void KMEDIA::cdMode()
{
  static uint32 status_old=~0;
  uint32 status;
  char   *modetext;
  char   modetextExited[]="Exited";
  char   modetextInit[]="Init";
  char   modetextBusy[]="Busy";
  char   modetextPlaying[]="Playing";
  char   modetextReady[]="Ready";
  char   modetextUnknown[]="Unknown";

  status = *StatStatPtr;

  if (status != status_old)
    {
      if ( status & MD_STAT_EXITED )
	modetext = modetextExited;
      else if ( status & MD_STAT_INIT )
	modetext = modetextInit;
      else if ( status & MD_STAT_BUSY )
	modetext = modetextBusy;
      else if ( status & MD_STAT_PLAYING )
	modetext = modetextPlaying;
      else if ( status & MD_STAT_READY )
	modetext = modetextReady;
      else modetext = modetextUnknown;

      /* Display changes in mode now. */
      statusPB->setText(modetext);
    }
}

void KMEDIA::setLCDColor( QLCDNumber *lcd, const QColor& fgColor, 
			const QColor& bgColor )
{

	QColorGroup old_cg = lcd->colorGroup();

	if ( lcd ) {
		QColorGroup new_cg( fgColor, bgColor,
				    fgColor, fgColor, fgColor, 
				    fgColor, fgColor );
		lcd->setPalette(QPalette(new_cg, new_cg, new_cg));
    	}
}

void KMEDIA::dtfatal(char *text)
{
  fprintf(stderr, "%s", text);
  exit(-1);
}

int main ( int argc, char *argv[] )
{
	QApplication 	kmedia( argc, argv );
	KMEDIA 		*k = new KMEDIA; 

	kmedia.setMainWidget( k );
	k->show();
	kmedia.exec();
}

